"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installSupply = exports.deletePrinter = exports.updatePrinter = exports.addPrinter = exports.getPrinterWithHistoryById = exports.getPrinterById = exports.getAllPrinters = void 0;
const printer_entity_1 = require("../entities/printer.entity");
const store_entity_1 = require("../entities/store.entity");
const error_handler_1 = require("../utils/error-handler");
const data_source_1 = require("../utils/data-source");
const getAllPrinters = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const fetchedPrinters = yield data_source_1.AppDataSource
            .createQueryBuilder()
            .where('printer.id IS NOT NULL')
            .select('printer')
            .from(printer_entity_1.PrinterEntity, 'printer')
            .leftJoinAndSelect('printer.supplies', 'stores')
            .leftJoinAndSelect('stores.supply', 'supply')
            .getMany();
        const printers = fetchedPrinters.map(printer => {
            return Object.assign(Object.assign({}, printer), { supplies: printer.supplies.map(storeItem => {
                    const { supply } = storeItem;
                    return {
                        name: supply.name,
                        installedAt: storeItem.installedAt,
                    };
                }) });
        });
        res.json({
            msg: 'Succeed',
            data: [...printers],
        });
    }
    // @ts-ignore
    catch (e) {
        const er = new error_handler_1.ValidationError(e.message);
        next(er);
    }
});
exports.getAllPrinters = getAllPrinters;
const getPrinterById = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { id } = req.params;
        const fetchedPrinter = yield printer_entity_1.PrinterEntity.findOneBy({ id });
        if (fetchedPrinter === null)
            return next(new error_handler_1.ValidationError('the printer was not found.'));
        res.json({
            msg: 'Succeed',
            data: [fetchedPrinter]
        });
    }
    catch (e) {
        const er = new error_handler_1.ValidationError(e.message);
        next(er);
    }
});
exports.getPrinterById = getPrinterById;
const getPrinterWithHistoryById = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { id } = req.params;
    try {
        const fetchedPrinter = yield data_source_1.AppDataSource
            .createQueryBuilder()
            .where('printer.id = :id', { id })
            .select('printer')
            .from(printer_entity_1.PrinterEntity, 'printer')
            .leftJoinAndSelect('printer.supplies', 'stores')
            .leftJoinAndSelect('stores.supply', 'supply')
            .getOne();
        //check if a printer exists.
        if (fetchedPrinter === null)
            return next(new error_handler_1.ValidationError('the printer was not found.'));
        const printer = Object.assign(Object.assign({}, fetchedPrinter), { supplies: fetchedPrinter.supplies.map(storeItem => {
                const { supply } = storeItem;
                return {
                    storeId: supply.id,
                    name: supply.name,
                    installedAt: storeItem.installedAt,
                };
            }) });
        res.json({
            msg: 'Succeed',
            data: [printer]
        });
    }
    catch (e) {
        const er = new error_handler_1.ValidationError(e.message);
        next(er);
    }
});
exports.getPrinterWithHistoryById = getPrinterWithHistoryById;
const addPrinter = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { name, ip, area, isMultifunctional, location, model } = req.body;
    try {
        const printerToAdd = new printer_entity_1.PrinterEntity();
        printerToAdd.name = name;
        printerToAdd.ip = ip;
        printerToAdd.model = model;
        printerToAdd.isMultifunctional = isMultifunctional;
        printerToAdd.area = area;
        printerToAdd.location = location;
        yield printerToAdd.save();
        res.json({
            msg: 'Succeed',
            data: [printerToAdd],
        });
    }
    catch (e) {
        const er = new error_handler_1.ValidationError(e.message);
        next(er);
    }
});
exports.addPrinter = addPrinter;
const updatePrinter = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { name, ip, area, isMultifunctional, location, model } = req.body;
    const { id } = req.params;
    // sends error when id param is undefined.
    // @todo add validation of id param. It should return fail respond message when id param is undefined.
    if (!id)
        return res.json({ msg: 'Fails' });
    const fetchedPrinter = yield printer_entity_1.PrinterEntity.findOne({ where: { id } });
    // sends error when printer was not found.
    if (fetchedPrinter === null)
        return next(new error_handler_1.ValidationError('the printer was not found.'));
    try {
        fetchedPrinter.name = name;
        fetchedPrinter.ip = ip;
        fetchedPrinter.model = model;
        fetchedPrinter.isMultifunctional = isMultifunctional;
        fetchedPrinter.area = area;
        fetchedPrinter.location = location;
        yield fetchedPrinter.save();
        // @todo add validation
        // @todo can not change model if the printer has already been supplied.
        res.json({
            msg: 'Succeed',
            data: [fetchedPrinter],
        });
    }
    catch (e) {
        const er = new error_handler_1.ValidationError(e.message);
        next(er);
    }
});
exports.updatePrinter = updatePrinter;
const deletePrinter = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { id } = req.params;
    // sends error when id param is undefined.
    if (!id)
        return res.json({ msg: 'Fails' });
    const fetchedPrinter = yield printer_entity_1.PrinterEntity.findOne({
        where: { id }, relations: {
            supplies: true,
        }
    });
    if (fetchedPrinter === null)
        return next(new error_handler_1.ValidationError('the printer was not found.'));
    //if printer has installed supplies, we have to remove the supplies from the store.
    yield data_source_1.AppDataSource
        .createQueryBuilder()
        .delete()
        .from(store_entity_1.StoreEntity)
        .where('printerId = :id', { id: fetchedPrinter.id })
        .execute();
    yield fetchedPrinter.remove();
    res.json({
        msg: 'Succeed',
        data: [],
    });
});
exports.deletePrinter = deletePrinter;
const installSupply = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { supplyId, printerId, } = req.params;
    try {
        const [availableSupply] = yield store_entity_1.StoreEntity.find({
            relations: {
                supply: true,
                printer: true,
            },
            where: {
                supply: {
                    id: supplyId,
                },
                isAvailable: true,
            }
        });
        if (availableSupply === undefined)
            return next(new error_handler_1.ValidationError('the supply is not available.'));
        const [printer] = yield printer_entity_1.PrinterEntity.find({
            relations: {
                supplies: true,
            },
            where: {
                id: printerId,
            }
        });
        if (printer === undefined)
            return next(new error_handler_1.ValidationError('Invalid printer id.'));
        //cannot install an unsuitable supply.
        if (printer.model !== availableSupply.supply.model)
            return next(new error_handler_1.ValidationError('can not install the unsuitable supply.'));
        availableSupply.printer = printer;
        availableSupply.isAvailable = false;
        availableSupply.installedAt = new Date();
        yield availableSupply.save();
        printer.supplies.push(availableSupply);
        yield printer.save();
        res.json({ msg: `${availableSupply.supply.name} has been installed at ${printer.name} printer.` });
    }
    catch (e) {
        next(e);
    }
});
exports.installSupply = installSupply;
//# sourceMappingURL=printer.controller.js.map