"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteItemFromStore = exports.updateItemInStore = exports.addItemToStore = exports.getItemById = exports.getItemsFromStore = void 0;
const store_entity_1 = require("../entities/store.entity");
const supply_entity_1 = require("../entities/supply.entity");
const error_handler_1 = require("../utils/error-handler");
const store_1 = require("../utils/store");
const getItemsFromStore = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
});
exports.getItemsFromStore = getItemsFromStore;
const getItemById = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { storeId } = req.params;
    try {
        const fetchStoreItem = yield store_entity_1.StoreEntity.findOne({
            where: { id: storeId },
            relations: {
                supply: true,
            },
        });
        if (fetchStoreItem === null)
            return next(new error_handler_1.ValidationError('the item can not be found in the database.'));
        res.json({
            msg: 'Succeed',
            data: [{
                    id: fetchStoreItem.id,
                    name: fetchStoreItem.supply.name,
                    model: fetchStoreItem.supply.model,
                    storedAt: fetchStoreItem.storedAt,
                    installedAt: fetchStoreItem.installedAt,
                    isAvailable: fetchStoreItem.isAvailable,
                }]
        });
    }
    catch (e) {
        next(e);
    }
});
exports.getItemById = getItemById;
const addItemToStore = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { supplyId } = req.params;
        const { quantity } = req.body;
        if (quantity === undefined)
            return next(new error_handler_1.ValidationError('Quantity must be defined.'));
        if (isNaN(quantity))
            return next(new error_handler_1.ValidationError('Quantity must be a number.'));
        if (quantity < 1)
            return next(new error_handler_1.ValidationError('Quantity must be higher then 0.'));
        if (quantity > 20)
            return next(new error_handler_1.ValidationError('Quantity must be lower then 20.'));
        const supply = yield supply_entity_1.SupplyEntity.findOneBy({ id: supplyId });
        if (supply === null)
            return next(new error_handler_1.ValidationError('Supply id was invalid.'));
        const newItemsToStore = (0, store_1.createStoreItems)(quantity);
        yield Promise.all(newItemsToStore.map((item) => __awaiter(void 0, void 0, void 0, function* () {
            item.supply = supply;
            yield item.save();
        })));
        res.json({ msg: `added ${quantity} - ${supply.name} for ${supply.model}.` });
    }
    catch (e) {
        console.log(e);
        next(e);
    }
});
exports.addItemToStore = addItemToStore;
const updateItemInStore = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
});
exports.updateItemInStore = updateItemInStore;
const deleteItemFromStore = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { storeId } = req.params;
    try {
        const fetchedStoreItem = yield store_entity_1.StoreEntity.findOneBy({ id: storeId });
        if (fetchedStoreItem === null)
            return next((new error_handler_1.ValidationError('the item can not be found in the store.')));
        if (fetchedStoreItem.installedAt !== null)
            return next((new error_handler_1.ValidationError('An Installed supply can not be deleted.')));
        yield fetchedStoreItem.remove();
        res.json({ msg: `item has been deleted.` });
    }
    catch (e) {
        console.log(e);
        next(e);
    }
});
exports.deleteItemFromStore = deleteItemFromStore;
//# sourceMappingURL=store.controller.js.map