"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSuppliesByModel = exports.getAvailableSupplies = exports.deleteSupply = exports.updateSupply = exports.addSupply = exports.getSupplyById = exports.getAllSupplies = void 0;
const supply_entity_1 = require("../entities/supply.entity");
const error_handler_1 = require("../utils/error-handler");
const supply_1 = require("../utils/supply");
const getAllSupplies = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const SuppliesList = yield supply_entity_1.SupplyEntity.find({
        select: ['id', 'name', 'model'],
    });
    const allSuppliesWithAvailability = yield (0, supply_1.checkSupplyAvailability)(SuppliesList);
    const categorizedSupplies = (0, supply_1.groupSuppliesByModel)(allSuppliesWithAvailability);
    res.json({
        msg: 'Succeed',
        data: [...categorizedSupplies],
    });
});
exports.getAllSupplies = getAllSupplies;
const getSupplyById = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { id } = req.params;
    if (!id)
        return res.json({ msg: 'Fails' });
    const fetchedSupply = yield supply_entity_1.SupplyEntity.findOneBy({ id });
    if (fetchedSupply === null)
        return next(new error_handler_1.ValidationError('the supply was not found.'));
    res.json({
        msg: 'Succeed',
        data: [fetchedSupply],
    });
});
exports.getSupplyById = getSupplyById;
const addSupply = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { name, model, code, } = req.body;
    try {
        const supplyToAdd = new supply_entity_1.SupplyEntity();
        supplyToAdd.name = name;
        supplyToAdd.code = code;
        supplyToAdd.model = model;
        yield supplyToAdd.save();
        res.json({ msg: 'Succeed', data: [supplyToAdd] });
    }
    catch (e) {
        next(e);
    }
});
exports.addSupply = addSupply;
const updateSupply = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { name, model, code, printerId } = req.body;
    const { id } = req.params;
    // @todo add validation of id param. It should return fail respond message when id param is undefined.
    if (!id)
        return res.json({ msg: 'Fails' });
    try {
        const fetchedSupply = yield supply_entity_1.SupplyEntity.findOneByOrFail({ id });
        fetchedSupply.name = name;
        //@todo valid model
        fetchedSupply.model = model;
        fetchedSupply.code = code;
        yield fetchedSupply;
        res.json({ msg: 'Succeed', data: [fetchedSupply] });
    }
    catch (e) {
        //@todo figure out how to handle validation messages.
        next(e);
    }
});
exports.updateSupply = updateSupply;
const deleteSupply = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { id } = req.params;
    // @todo add validation of id param. It should return fail respond message when id param is undefined.
    if (!id)
        return res.json({ msg: 'Fails' });
    try {
        // @todo change finding method to simply findOneBy.
        const fetchedSupply = yield supply_entity_1.SupplyEntity.findOneByOrFail({ id });
        yield fetchedSupply.remove();
        res.json({ msg: 'Succeed', data: [] });
    }
    catch (e) {
        //@todo figure out how to handle validation messages.
        next(e);
    }
});
exports.deleteSupply = deleteSupply;
const getAvailableSupplies = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { model } = req.params;
    if (!(0, supply_1.isAccurateModel)(model))
        return next(new error_handler_1.ValidationError('model was invalid.'));
    const suppliesList = yield (0, supply_1.listSuppliesForPrinter)(model);
    const suppliesListWithTotals = yield (0, supply_1.checkSupplyAvailability)(suppliesList);
    res.json({
        msg: 'Succeed',
        data: suppliesListWithTotals
    });
});
exports.getAvailableSupplies = getAvailableSupplies;
const getSuppliesByModel = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { model } = req.params;
    if (!(0, supply_1.isAccurateModel)(model))
        return next(new error_handler_1.ValidationError('model was invalid.'));
    const suppliesList = yield (0, supply_1.listSuppliesForPrinter)(model);
    res.json({
        msg: 'Succeed',
        data: suppliesList
    });
});
exports.getSuppliesByModel = getSuppliesByModel;
//# sourceMappingURL=supply.controller.js.map