"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.groupSuppliesByModel = exports.checkSupplyAvailability = exports.listSuppliesForPrinter = exports.isAccurateModel = void 0;
const index_1 = require("../../types/index");
const supply_entity_1 = require("../entities/supply.entity");
const store_entity_1 = require("../entities/store.entity");
const isAccurateModel = (model) => {
    const models = Object.values(index_1.PrinterModel);
    return models.includes(model);
};
exports.isAccurateModel = isAccurateModel;
const listSuppliesForPrinter = (model) => __awaiter(void 0, void 0, void 0, function* () {
    return supply_entity_1.SupplyEntity.find({
        where: {
            model,
        },
        select: {
            id: true,
            name: true,
        },
        order: {
            code: 'ASC',
        }
    });
});
exports.listSuppliesForPrinter = listSuppliesForPrinter;
const checkSupplyAvailability = (listOfSupplies) => __awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(listOfSupplies.map((supply) => __awaiter(void 0, void 0, void 0, function* () {
        const total = yield store_entity_1.StoreEntity.countBy({
            supply: {
                id: supply.id,
            },
        });
        const totalAvailable = yield store_entity_1.StoreEntity.countBy({
            supply: {
                id: supply.id,
            },
            isAvailable: true,
        });
        return {
            id: supply.id,
            name: supply.name,
            model: supply.model,
            total,
            totalAvailable,
        };
    })));
});
exports.checkSupplyAvailability = checkSupplyAvailability;
const groupSuppliesByModel = (listOfSupplies) => {
    const models = (Object.values(index_1.PrinterModel));
    const results = models.map((model) => {
        return {
            model,
            supplies: listOfSupplies.filter(supply => supply.model === model),
        };
    });
    return results;
};
exports.groupSuppliesByModel = groupSuppliesByModel;
//# sourceMappingURL=supply.js.map